#ifndef GNTST_BASE_H
#define GNTST_BASE_H

/* size_t */
#if defined(__cplusplus)
#include <cstddef>
#else
#include <stddef.h>
#endif

#define NTST_STRINGIZE(x) #x
#define NTST_STRINGIZE_MACRO(x) NTST_STRINGIZE(x)

/* ntst_uint32_t */
#if defined(_MSC_VER) || defined(__BORLANDC__)
#define ntst_uint32_t unsigned __int32
#else
#include <stdint.h>
#define ntst_uint32_t uint32_t
#endif

#if defined(__cplusplus)
#   if (__cplusplus >= 201103L) || (defined(_MSC_VER) && (_MSC_VER >= 1900))
#       define NTST_CONSTEXPR constexpr
#   else
#       define NTST_CONSTEXPR
#   endif

template <typename ElemType, size_t arr_size>
NTST_CONSTEXPR size_t NTST_ARRAY_LEN(ElemType const (&)[arr_size])
    {
    return arr_size;
    }
#else
#   define NTST_ARRAY_LEN(array) (sizeof(array)/sizeof(*array))
#endif

#if defined(__cplusplus)
template <typename ElemType, size_t arr_size>
NTST_CONSTEXPR size_t NTST_STR_LEN(ElemType const (&)[arr_size])
    {
    return arr_size - 1;
    }
#else
#   define NTST_STR_LEN(array) (sizeof(array)/sizeof(*array) - 1)
#endif

/* NTST_NOEXCEPT */
#if defined(__cplusplus)
#   if (__cplusplus >= 201103L)
#       define NTST_NOEXCEPT noexcept
#   else
#       define NTST_NOEXCEPT throw()
#   endif
#else
#   define NTST_NOEXCEPT
#endif

/* Branch prediction */
#if defined(__GNUC__) || defined(__clang__)
#define NTST_LIKELY(expr) __builtin_expect(!!(expr),!0)
#define NTST_UNLIKELY(expr) __builtin_expect(!!(expr),0)
#else
/* cppcheck doesn't like (!!(expr)), NTST_LIKELY(tmp != NULL) == tmp dereference before check */
#define NTST_LIKELY(expr) (expr)
#define NTST_UNLIKELY(expr) (expr)
#endif

/* NTST_RESTRICT */
#if defined(_MSC_VER)
#   define NTST_RESTRICT __restrict
#elif defined(__GNUC__)
#   define NTST_RESTRICT __restrict__
#endif
#if defined(__STDC__)
#   if (__STDC_VERSION__ >= 199901L)
#       include <stdbool.h>
#       if !defined(NTST_RESTRICT)
#           define NTST_RESTRICT restrict
#       endif
#   endif
#endif
#if !defined(NTST_RESTRICT)
#   define NTST_RESTRICT
#endif
/* NTST_RESTRICT end */

/* static_assert */
#if (defined(__cplusplus) && (__cplusplus >= 201103L || defined(__GXX_EXPERIMENTAL_CXX0X__)))
#   define ntst_static_assert(a, b) static_assert(a, b)
#elif (defined(_MSC_VER)) && (_MSC_VER >= 1600)
#   define ntst_static_assert(a, b) static_assert(a, b)
#elif (defined(__STDC__) && (__STDC_VERSION__ >= 201112L))
#   define ntst_static_assert(a, b) _Static_assert(a, b)
#else
#   define ntst_static_assert(a, b)
#   define NTST_NO_STATIC_ASSERT 1
#endif

#if (defined(__cplusplus) && (__cplusplus >= 201103L))
#   define NTST_HAS_RVALUE_REF 1
#elif (_MSC_VER >= 1600)
#   define NTST_HAS_RVALUE_REF 1
#elif defined(__clang__)
#   if __has_feature(cxx_rvalue_references)
#       define NTST_HAS_RVALUE_REF 1
#   endif
#elif (__GNUC__ > 4 || __GNUC__ == 4 && __GNUC_MINOR__ >= 3) || defined(__GXX_EXPERIMENTAL_CXX0X__)
#   define NTST_HAS_RVALUE_REF 1
#endif


#if defined(__cplusplus)
/* final */
#if (__cplusplus >= 201103L)
#   define NTST_FINAL final
#elif (defined(_MSC_VER)) && (_MSC_VER >= 1700)
#   define NTST_FINAL final
#else
#   define NTST_FINAL
#endif
/* override */
#if (__cplusplus >= 201103L)
#   define NTST_OVERRIDE override
#elif (defined(_MSC_VER)) && (_MSC_VER >= 1700)
#   define NTST_OVERRIDE override
#else
#   define NTST_OVERRIDE
#endif
#define NTST_FINAL_OVERRIDE NTST_FINAL NTST_OVERRIDE
#endif /* defined(__cplusplus) */

#ifndef NTST_THROW_BAD_ALLOC
#define NTST_THROW_BAD_ALLOC throw std::bad_alloc()
#endif

/* #ifndef GNTST_BASE_H */
#endif
