#ifndef GNTST_BYTES_H
#define GNTST_BYTES_H

#include <stdint.h> /* uint.._t */
#include "ntst_base.h"

#ifdef __cplusplus
extern "C" {
#endif

uint16_t ntst_bytes_swap16(uint16_t val);
uint32_t ntst_bytes_swap32(uint32_t val);

size_t ntst_buf_cmp(const void* buf1, const void* buf2, size_t buf_size);

void ntst_memcopy_reversed(void* NTST_RESTRICT dest, const void* NTST_RESTRICT src, size_t count);

#ifdef __cplusplus
}
namespace ntst
{
template<typename SignedInt>
unsigned EncodeSLEB128(SignedInt val, void* buf)
    {
    unsigned char* start = (unsigned char*)buf;
    unsigned char* cur_ptr = (unsigned char*)buf;
    bool more;
    do
        {
        uint8_t byte = val & 0x7F;
        /* TODO need static check signed right shift is an arithmetic. */
        val >>= 7;
        const bool signBitIsClear = ((byte & 0x40) == 0);
        more = !(((val == 0) && signBitIsClear) ||
              ((val == -1) && !signBitIsClear));
        if (more)
            byte |= 0x80;
        if (buf != NULL)
            *cur_ptr = byte;
        ++cur_ptr;
        }
    while (more);
    return (cur_ptr - start);
    }

template<typename UnsignedInt>
unsigned EncodeULEB128(UnsignedInt val, void* buf)
    {
    unsigned char* start = (unsigned char*)buf;
    unsigned char* cur_ptr = (unsigned char*)buf;
    do
        {
        uint8_t byte = val & 0x7F;
        val >>= 7;
        if (val != 0)
            byte |= 0x80;
        if (buf != NULL)
            *cur_ptr = byte;
        ++cur_ptr;
        }
    while (val != 0);
    return (cur_ptr - start);
    }

template<typename SignedInt>
unsigned DecodeSLEB128(const void* buf, size_t buf_size, SignedInt& val)
    {
    val = 0;
    unsigned char* start = (unsigned char*)buf;
    unsigned char* cur_ptr = (unsigned char*)buf;
    unsigned shift = 0;
    unsigned size_bits = sizeof(val) * CHAR_BIT;
    unsigned char byte;
    for (;;)
        {
        if (buf_size == 0)
            return 0;
        byte = *cur_ptr;
        val |= ((SignedInt)(byte & 0x7F) << shift);
        ++cur_ptr;
        if (byte <= 0x7F)
            break;
        shift += 7;
        --buf_size;
        }
    if ((shift < size_bits) && (byte & 0x40))
        val |= -((SignedInt)1 << shift); /* sign extend */
    return (cur_ptr - start);
    }

template<typename UnsignedInt>
unsigned DecodeULEB128(const void* buf, size_t buf_size, UnsignedInt& val)
    {
    val = 0;
    unsigned char* start = (unsigned char*)buf;
    unsigned char* cur_ptr = (unsigned char*)buf;
    unsigned shift = 0;
    for (;;)
        {
        if (buf_size == 0)
            return 0;
        unsigned char byte = *cur_ptr;
        val |= ((UnsignedInt)(byte & 0x7F) << shift);
        ++cur_ptr;
        if (byte <= 0x7F)
            break;
        shift += 7;
        --buf_size;
        }
    return (cur_ptr - start);
    }
}
#endif

#endif
