#ifndef GNTST_STRING_H
#define GNTST_STRING_H

#include "ntst_base.h"

#include <stddef.h> /* size_t C/C++ */
#include <string.h>
#include <stdarg.h>

#define NTST_ONE_NULL_TERM 1

#ifdef __cplusplus
extern "C" {
#else
int ntst_strhasprefix(const char* NTST_RESTRICT string, const char* NTST_RESTRICT prefix);
#endif

int ntst_ascii_tolower(int ch);

/* '0'->0, 'F'->0xF, negative on failure */
int ntst_hex_char_to_int(char c);
/* '00'->0, 'FF'->0xFF, negative on failure(-1 fail high, -2 fail low) */
int ntst_hex_char_pair_to_int(char c_high, char c_low);
void ntst_byte_to2char(unsigned char byte, char* out);

/**
 * Под виндой переполнение это ошибка -1. По стандарту возвращает сколько было бы записано.
 */
int ntst_snprintf(char* NTST_RESTRICT buff, unsigned max_len, const char* NTST_RESTRICT format, ...)
#ifdef __GNUC__
__attribute__((format(printf,3,4)))
#endif
;
int ntst_vsnprintf(char* NTST_RESTRICT buff, unsigned max_len, const char* NTST_RESTRICT format, va_list vlist);

/**
  * Соединяет '\0' терминированные строки, пока не встретит NULL или не кончится max_len
  * Возвращает размер буфера нужный чтобы вместить все строки без терминатора.
  * Результат без терминатора!
  * buff == NULL можно если max_len == 0
  */
size_t ntst_str_concat(char* NTST_RESTRICT buff, size_t max_len, ...);

/**
 * @brief ntst_strncpy - Копирует не более count - 1 символов из src в dest и добавляет 0 терминатор, если count > 0.
 * Почему не strncpy:
 * If count is reached before the entire string src was copied, the resulting character array is not null-terminated.
 * Ненавижу тех кто иногда не пришмякивает нули.
 * If, after copying the terminating null character from src, count is not reached, additional null characters are written to dest until the total of count characters have been written.
 * Почти 4 килобайта нулей в пути к файлу в линуксе. Да за такое на помоечку сразу.
 * @param dest
 * @param src
 * @param count
 * @return Число скопированных символов без учета 0 терминатора(==strlen). Или 0 если count == 0
 */
size_t ntst_strncpy(char* NTST_RESTRICT dest, const char* NTST_RESTRICT src, size_t count);

size_t ntst_str10toi(const char* NTST_RESTRICT str, int* NTST_RESTRICT result);
size_t ntst_str10tol(const char* NTST_RESTRICT str, long* NTST_RESTRICT result);
size_t ntst_str10toll(const char* NTST_RESTRICT str, long long* NTST_RESTRICT result);
size_t ntst_str10tou(const char* NTST_RESTRICT str, unsigned* NTST_RESTRICT result);
size_t ntst_strn10toi(const char* NTST_RESTRICT str, size_t len, int* NTST_RESTRICT result);
size_t ntst_strn10tol(const char* NTST_RESTRICT str, size_t len, long* NTST_RESTRICT result);
size_t ntst_strn10tou(const char* NTST_RESTRICT str, size_t len, unsigned* NTST_RESTRICT result);
size_t ntst_strtof(const char* NTST_RESTRICT str, float* NTST_RESTRICT result);
size_t ntst_strtod(const char* NTST_RESTRICT str, double* NTST_RESTRICT result);

/**
  * Результат без нультерминатора. Поэтому str нужно минимум blob_size*2. +1 если ручками добавляем '\0'
  */
void ntst_blobtohexstr(const void* NTST_RESTRICT blob, size_t blob_size, char* str);
/**
  * '\'  - \\
  * '\0' - \00
  * '\a' - \a
  * '\b' - \b
  * '\t' - \t
  * '\n' - \n
  * '\v' - \v
  * '\f' - \f
  * '\r' - \r
  *  max result size blob_size*NTST_ESCAPED_CHAR_MAX_SIZE
  */
#define NTST_ESCAPED_CHAR_MAX_SIZE 3
size_t ntst_escape_char(char c, void* out);
size_t ntst_blobtoescapedstr(const void* NTST_RESTRICT blob, size_t blob_size, char* NTST_RESTRICT str);
/* String allocated with NTST_MALLOC, must be freed with NTST_FREE */
char* ntst_blob_to_str(const char* NTST_RESTRICT blob_name, const void* NTST_RESTRICT blob, size_t* NTST_RESTRICT size);

/**
 * @brief ntst_strchr_after_n пропускает num символов равных c и возвращает указатель на строку после последнего.
 * @return NULL если в строке нет num симолов c.
 */
char* ntst_strchr_after_n(const char* str, int c, unsigned num);
char* ntst_strnchr(const char* str, size_t n, int ch);
size_t ntst_strchr_cnt(const char* str, char c);

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif
