#include "ntst_thread.h"
#include "ntst_base.h"

#if defined(_WIN32)

ntst_error_t ntst_sleep_millisec(unsigned milliseconds)
    {
    if (NTST_LIKELY(SleepEx(milliseconds, TRUE) == 0))
        return NTST_ERROR_OK;
    return NTST_ERROR_EINTR;
    }

ntst_error_t ntst_thread_yield()
    {
    SwitchToThread();
    return NTST_ERROR_OK;
    }

#else

#include <features.h>
#include <time.h> /* nanosleep */
#include <sched.h>

#if _POSIX_C_SOURCE >= 199309L

ntst_error_t ntst_sleep_millisec(unsigned milliseconds)
    {
    struct timespec ts;
    ts.tv_sec = milliseconds / 1000;
    ts.tv_nsec = (long)(milliseconds % 1000) * 1000000;
    /* NULL, the remaining time is not returned */
    if (NTST_LIKELY(nanosleep(&ts, NULL) == 0))
        return NTST_ERROR_OK;
    return ntst_error_from_errno(errno);
    }
#else
#include <unistd.h> /* usleep */
/* usleep is obsolete */
ntst_error_t ntst_sleep_millisec(unsigned milliseconds)
    {
    if (NTST_LIKELY(usleep((useconds_t)milliseconds * 1000) == 0))
        return NTST_ERROR_OK;
    return ntst_error_from_errno(errno);
    }
#endif /* _POSIX_C_SOURCE >= 199309L */

ntst_error_t ntst_thread_yield()
    {
    if (NTST_LIKELY(sched_yield() == 0))
        return NTST_ERROR_OK;
    return ntst_error_from_errno(errno);
    }

#endif /* defined(_WIN32) */
